

function draw_grid(r,g,b,a,u)
  for i = 0, window.w/u do
    for j = 0, window.h/u do
      love.graphics.setColor(r,g,b,a)
      love.graphics.rectangle("line", i*u, j*u, u, u)
      love.graphics.setColor(1,1,1,1)
    end
  end
end

function draw_console(t) --t == console string
  love.graphics.setFont(font[2])
  if t == "mouse" or t == "pos" then
    love.graphics.setColor(1,1,1,1)
    love.graphics.rectangle("fill", 0, 0, window.w, console_h)
    love.graphics.setColor(0,0,0,1)
    love.graphics.print(">: mouse X: "..mouse.x.." unit X: "..math.floor(mouse.x/(u*0.5)),10,0)
    love.graphics.print("mouse Y: "..mouse.y.." unit Y: "..math.floor(mouse.y/(u*0.5)),u*4,0)
    love.graphics.setColor(1,1,1,1)
  elseif t == "folder" or t == "dir" then
    local dir = love.filesystem.getSaveDirectory( )
    love.graphics.setColor(1,1,1,1)
    love.graphics.rectangle("fill", 0, 0, window.w, console_h)
    love.graphics.setColor(0,0,0,1)
    love.graphics.print(dir, 10, 0)
    love.graphics.setColor(1,1,1,1)
  else
    love.graphics.setColor(1,1,1,1)
    love.graphics.rectangle("fill", 0, 0, window.w, console_h)
    love.graphics.setColor(0,0,0,1)
    love.graphics.print(">: ",10, 0)
    love.graphics.setColor(1,1,1,1)
  end
  love.graphics.setFont(font[1])
end

function draw_canvas(canvas,x,y,sx,sy)
  if canvas then
    -- very important!: reset color before drawing to canvas to have colors properly displayed
    love.graphics.setColor(1, 1, 1, 1)
    -- Use the premultiplied alpha blend mode when drawing the Canvas
    love.graphics.setBlendMode("alpha", "premultiplied")
      love.graphics.draw(canvas,x,y,0,sx,sy)
    love.graphics.setBlendMode("alpha", "alphamultiply")
    --print out the file path save directory of expoerted image
  end
end

function draw_spriteBackground(_x,_y,r,g,b,a)
  local r = r or 1
  local g = g or 0
  local b = b or 1
  local a = a or 1
  love.graphics.setColor(r,g,b,a)
  love.graphics.rectangle("fill", _x, _y, 64*4, 128*4)
  love.graphics.setColor(1,1,1,1)
end

function draw_spriteOverlay(_x,_y,r,g,b,a)
  local r = r or 160/255
  local g = g or 110/255
  local b = b or 88/255
  local a = a or 255/255
  love.graphics.setColor(r,g,b,a)
  love.graphics.setBlendMode("multiply", "premultiplied")
  --love.graphics.rectangle("fill", _x, _y, 64*4, 128*4)
  skin.animation[sprite.select]:draw(skin.base,_x, _y,0,4,4)
  love.graphics.setBlendMode("alpha")
  love.graphics.setColor(1,1,1,1)
end

function draw_anim8Background(_x,_y,r,g,b,a)
  local r = r or 1
  local g = g or 0
  local b = b or 1
  local a = a or 1
  love.graphics.setColor(r,g,b,a)
  love.graphics.rectangle("fill", _x, _y, 64*1, 128*4)
  love.graphics.setColor(1,1,1,1)
end

function draw_anim8Overlay(_x,_y,r,g,b,a)
  local r = r or 160/255
  local g = g or 110/255
  local b = b or 88/255
  local a = a or 255/255
  love.graphics.setColor(r,g,b,a)
  love.graphics.setBlendMode("multiply", "premultiplied")
  for i = 0, 3 do
    skin.animation[bodyDirection[i+1]]:draw(skin.base,_x, _y + i*128)
  end
  love.graphics.setBlendMode("alpha")
  love.graphics.setColor(1,1,1,1)
end

function draw_faceBackground(_x,_y,r,g,b,a)
  local r = r or 1
  local g = g or 0
  local b = b or 1
  local a = a or 1
  love.graphics.setColor(r,g,b,a)
  love.graphics.rectangle("fill", _x , _y, face.w*2, face.h*2)
  love.graphics.setColor(1,1,1,1)
end

function draw_faceOverlay(_x,_y,r,g,b,a)
  local r = r or 160/255
  local g = g or 110/255
  local b = b or 88/255
  local a = a or 255/255
  love.graphics.setColor(r,g,b,a)
  love.graphics.setBlendMode("multiply", "premultiplied")
  love.graphics.rectangle("fill", _x , _y, face.w*2, face.h*2)
  love.graphics.setBlendMode("alpha")
  love.graphics.setColor(1,1,1,1)
end

function draw_sprite(x,y)
    local dx,dy = sprite_x, sprite_y
    local _x = x or dx
    local _y = y or dy
    draw_spriteBackground(_x,_y)
    for i = 0, 3 do
      for j = 0, 3 do
        love.graphics.rectangle("line", _x + i*64, _y + j*128, 64, 128)
      end
    end
    draw_canvas(sprite.canvas, _x, _y)
end

function draw_anim8sprite(x,y)
  local dx,dy = anim8sprite_x, anim8sprite_y
  local _x = x or dx
  local _y = y or dy
  draw_anim8Background(_x,_y)
  for i = 0, 3 do
    love.graphics.rectangle("line", _x, _y + i*128, 64, 128)
    sprite.animation[i+1]:draw(sprite.base,_x, _y + sprite.h*i)
  end
end

function draw_bigsprite(state,x,y)
  local dx,dy = sprite_x, sprite_y
  local _x = x or dx
  local _y = y or dy
  draw_spriteBackground(_x,_y)
  if state == "front" then
    sprite.animation[1]:draw(sprite.base,_x, _y,0,4,4)
  elseif state == "left" then
    sprite.animation[2]:draw(sprite.base,_x, _y,0,4,4)
  elseif state == "right" then
    sprite.animation[3]:draw(sprite.base,_x, _y,0,4,4)
  elseif state == "back" then
    sprite.animation[4]:draw(sprite.base,_x, _y,0,4,4)
  end
end

function draw_body(x,y)
    local dx,dy = sprite_x, sprite_y
    local _x = x or dx
    local _y = y or dy
    draw_spriteBackground(_x,_y)
    for i = 0, 3 do
      for j = 0, 3 do
        love.graphics.rectangle("line", _x + i*64, _y + j*128, 64, 128)
      end
    end
    love.graphics.draw(body.base, _x, _y)
end

function draw_anim8body(x,y)
  local dx,dy = anim8sprite_x, anim8sprite_y
  local _x = x or dx
  local _y = y or dy
  draw_anim8Background(_x,_y)
  for i = 0, 3 do
    love.graphics.rectangle("line", _x, _y + i*128, 64, 128)
    body.animation[i+1]:draw(body.base,_x, _y + body.h*i)
  end
end

local function draw_faces(_x, _y)
  local fx1, fy1 = _x, _y
  love.graphics.rectangle("line", fx1 , fy1, face.w, face.h)
  love.graphics.draw(face.profile[1], fx1, fy1)
  local fx2, fy2 = _x + 72, _y
  love.graphics.rectangle("line", fx2 , fy2, face.w, face.h)
  love.graphics.draw(face.profile[2], fx2, fy2)
  local fx3, fy3 = _x, _y + 72
  love.graphics.rectangle("line", fx3 , fy3, face.w, face.h)
  love.graphics.draw(face.profile[3], fx3, fy3)
  local fx4, fy4 = _x + 72, _y + 72
  love.graphics.rectangle("line", fx4 , fy4, face.w, face.h)
  love.graphics.draw(face.profile[4], fx4, fy4)
end

function draw_face(x,y)
    local dx,dy = face_x, face_y
    local _x = x or dx
    local _y = y or dy
    draw_faceBackground(_x, _y)
    if labels.state[1] then
      draw_faces(_x, _y)
    else --draw single large face
    love.graphics.rectangle("line", _x , _y, face.w*2, face.h*2)
    draw_canvas(face.canvas, _x, _y,2,2) --scale by 2
    end
end

function draw_labels(x,y)
  local dx,dy = labels_x, labels_y
  local _x = x or dx
  local _y = y or dy
  local div_y = labels_divY
  love.graphics.setColor(0,0,0,1)
  love.graphics.rectangle("fill", _x, _y, labels.w, labels.h*(#labels + 1)+u/6)
  love.graphics.setColor(1,1,1,1)
  for i = 0, (#labels - 1) do
    if labels.select == labels[i+1] then labels.mode = "fill" else labels.mode = "line" end
      love.graphics.rectangle(labels.mode, _x, _y + (i*(u/div_y)), labels.w, labels.h,u/8,u/8,u)
    if labels.select == labels[i+1] then --set font color to contrast
    love.graphics.setColor(0,0,0,1)  else love.graphics.setColor(1,1,1,1)
    end
      love.graphics.print(labels[i+1], _x + (labels.w/4), _y + (i*(u/div_y)))
      love.graphics.setColor(1,1,1,1)
  end
end

function draw_testinput(x)
  local dx = test_x
  local _x = x or dx
  for i = 1, #labels do
    love.graphics.print("label:"..i.." "..tostring(labels.state[i]),_x,i*(u/2))
  end
  love.graphics.print("label selection: "..labels.select,_x,(#labels + 1)*(u/2))
  love.graphics.print("sprite selection: "..sprite.select,_x,(#labels + 2)*(u/2))
  love.graphics.print("body selection: "..body.select,_x,(#labels + 3)*(u/2))
  love.graphics.print("face index: "..face.index,_x,(#labels + 4)*(u/2))
end

function print_texturetypes()
  textypes = love.graphics.getTextureTypes()
  local y = 0
  for name, supported in pairs(textypes) do
      local str = string.format("Supports texture type '%s': %s", name, tostring(supported))
      love.graphics.print(str, 10, y)
      y = y + 20
  end
end
